/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.docware;

import java.util.Set;
import java.util.HashSet;

/**
 * A <code>Profile</code> is a collection of <code>Descriptors</code>.
 */

public class Profile {

    private Set<Descriptor> descriptors_ = new HashSet<Descriptor>();

    // ------------------------------------------------------------
    public void add(Descriptor desc) {
	descriptors_.add(desc);
    }

    // ------------------------------------------------------------
    public void remove(Descriptor desc) {
	descriptors_.remove(desc);
    }

    // ------------------------------------------------------------
    public int size() {
	return descriptors_.size();
    }

    // ------------------------------------------------------------
    public Set<Descriptor> descriptors() {
	return descriptors_;
    }

    // ------------------------------------------------------------
    public void clear() {
	descriptors_.clear();
    }

    // ------------------------------------------------------------
    /**
     * @undocumented
     *
     * Produces a <code>String</code> representation of this
     * <code>Profile</code>.
     * @return a <code>String</code> representation of this
     *         <code>Descriptor</code>.
     */
    @Override
	public String toString() {

	String result = "<profile> ";
	for (Descriptor desc: descriptors_) {
	    result += desc + " ";
	}
	result += "</profile>";

	return result;
    }

    // ============================================================
    /**
     * @undocumented
     */
    public static void main(String[] args) throws Exception {

	Profile prof1 = new Profile();
	ProfileBufferizer cb = new ProfileBufferizer();

	Descriptor d1 = new Descriptor();
	d1.setAttribute("att1", "fg");
	d1.setAttribute("att2", "rg jg");
	d1.setAttribute("att3", "pg");
	Descriptor d2 = new Descriptor();
	d2.setAttribute("att1", "f.*");
	d2.setAttribute("att2", ".*rg.*");

	prof1.add(d1);
	prof1.add(d2);

	cb.toFile(prof1, "prof1");

	Profile prof2 = cb.fromFile("prof1");
	cb.toFile(prof2, "prof2");
    }

}
