/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.net;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.MulticastSocket;
import java.net.NetworkInterface;
import java.util.Random;

import casa.dodwan.util.ArgumentNotFoundException;
import casa.dodwan.util.ArgumentParsing;
import casa.dodwan.util.ArgumentParsingException;
import casa.dodwan.util.Command;

public class sendUDP 
    extends Command {

    private static DatagramSocket socket_ = null;
    private static Random random_ = new Random(System.currentTimeMillis());


    //------------------------------------------------------------
    private static byte checksum(byte[] buffer, int offset, int length) {
	
	byte checksum = 0;
	for (int i=offset; i<offset + length; i++) {
	    checksum += buffer[i];
	}
	return checksum;
    }
    
    //------------------------------------------------------------
    private static void randomize(byte[] buffer) {
	
	for (int i=1; i<buffer.length; i++) {
	    buffer[i] = (byte)(random_.nextInt(26) + 65);
	}
	
	buffer[0] = checksum(buffer, 1, buffer.length - 1);
    }

   // ------------------------------------------------------------
    public static void sendPacket(int length, 
			    InetAddress remoteAddress, 
			    int remotePort) 
	throws Exception {
	
	byte[] buffer = new byte[length];
	randomize(buffer);
	
	DatagramPacket packet =
	    new DatagramPacket(buffer, buffer.length, 
			       remoteAddress, remotePort);
	socket_.send(packet);

	byte checksum = checksum(buffer, 1, length-1); 
	System.out.println("-> Packet sent to "
			   + packet.getAddress()
			   + ":" + packet.getPort());
	System.out.println("   length= " + length
			   + ", checksum= " + buffer[0]
			   + " (expected= " + checksum + ")");
    }

    // ------------------------------------------------------------
    public static void help() {
        System.out
                .println("\n"
			 + "Syntax: sendUDP [-a address] [-p port] [-r repeat] [-w time]\n"
			 + "                [-min value] [-max value] [-step value]");
    }
    
    // ------------------------------------------------------------
    public static void main(String[] args) throws Exception {
	
        if ((args.length == 0)
                || (ArgumentParsing.searchIndex("-h", args) != -1)) {
	    help();
            exit(0);
        }

	String[] options = args;

	//----------------------------------------
        String address = null;
        int port = 0;
	int repeat = 1;
	int wait = 0;
        int min = 1;
        int max = 1;
	int step = 1;

        int index;

        try {
            address = ArgumentParsing.searchString("-a", options);
            port    = ArgumentParsing.searchInt("-p", options);
            repeat  = ArgumentParsing.searchInt("-r", options);
            wait    = ArgumentParsing.searchInt("-w", options);
            min     = ArgumentParsing.searchInt("-min", options);
            max     = ArgumentParsing.searchInt("-max", options);
            step    = ArgumentParsing.searchInt("-step", options);
        } catch (ArgumentNotFoundException e) {
            missingArgument(e);
            exit(1);
        } catch (ArgumentParsingException e) {
            parsingException(e);
            exit(1);
        }

	InetAddress remoteAddress = InetAddress.getByName(address);

	socket_ = new DatagramSocket();

	for (int length = min; length <= max; length += step)
	    for (int n=0; n<repeat; n++) {
		sendPacket(length, remoteAddress, port);
		Thread.sleep((long)wait);
	    }

	socket_.close();
    }
}
