/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

public class DataBufferizer {

    private static char[] digits = {'0','1','2','3','4','5','6','7','8','9',
        'a','b','c','d','e','f'};

	public static void copyBuffer(byte[] sourceBuffer, int sourceOffset,
			byte[] toBuffer, int toOffset, int length) {
		for (int i = 0; i < length; i++)
			toBuffer[toOffset + i] = sourceBuffer[sourceOffset + i];
	}

	public static void displayBuffer(byte[] buffer) {
		displayBuffer(buffer, 0, buffer.length);
	}

	public static void displayBuffer(byte[] buffer, int offset, int length) {
		for (int i = 0; i < length; i++) {
			System.out.print(buffer[i + offset] + " ");
		}
		System.out.println();
	}

	public static void main(String[] args) throws Exception {

		byte[] buffer = new byte[128];

		displayBuffer(buffer);
		writeBoolean(true, buffer, 0);
		writeBoolean(false, buffer, 1);
		writeBoolean(true, buffer, 2);
		writeByte(12, buffer, 3);
		writeByte(-37, buffer, 4);
		writeChar(71, buffer, 5);
		// writeInt(543, buffer, 7);
		writeInt(123456789, buffer, 7);
		writeInt(-123456789, buffer, 11);
		writeLong(Long.MAX_VALUE - 123456, buffer, 15);
		writeLong(Long.MIN_VALUE + 123456, buffer, 23);
		writeFloat((float) 3.1415926, buffer, 31);
		writeFloat((float) -3.1415926, buffer, 35);
		writeDouble(3.1415926, buffer, 39);
		writeDouble(-3.1415926, buffer, 47);
		writeShort(12345, buffer, 55);
		writeShort(-12345, buffer, 57);
		writeString("This is a test. Nothing but a test...", buffer, 59);

		displayBuffer(buffer);

		System.out.println(readBoolean(buffer, 0));
		System.out.println(readBoolean(buffer, 1));
		System.out.println(readBoolean(buffer, 2));
		System.out.println(readByte(buffer, 3));
		System.out.println(readByte(buffer, 4));
		System.out.println(readChar(buffer, 5));
		System.out.println(readInt(buffer, 7));
		System.out.println(readInt(buffer, 11));
		System.out.println(readLong(buffer, 15) + " ("
				+ (Long.MAX_VALUE - 123456) + ")");
		System.out.println(readLong(buffer, 23) + " ("
				+ (Long.MIN_VALUE + 123456) + ")");
		System.out.println(readFloat(buffer, 31));
		System.out.println(readFloat(buffer, 35));
		System.out.println(readDouble(buffer, 39));
		System.out.println(readDouble(buffer, 47));
		System.out.println(readShort(buffer, 55));
		System.out.println(readShort(buffer, 57));
		System.out.println(readString(buffer, 59));
	}

	public static boolean readBoolean(byte[] buffer, int offset)
			throws BufferAccessException {
		return (buffer[offset] != 0);
	}

	public static byte readByte(byte[] buffer, int offset)
			throws BufferAccessException {
		return buffer[offset];
	}

	public static char readChar(byte[] buffer, int offset)
			throws BufferAccessException {
		int val = buffer[offset] << 8;
		val |= buffer[offset + 1];
		return ((char) val);
	}

	public static double readDouble(byte[] buffer, int offset)
			throws BufferAccessException {
		return Double.longBitsToDouble(readLong(buffer, offset));
	}

	public static float readFloat(byte[] buffer, int offset)
			throws BufferAccessException {
		return Float.intBitsToFloat(readInt(buffer, offset));
	}

	public static int readInt(byte[] buffer, int offset)
			throws BufferAccessException {
		int value = (buffer[offset] & 0xFF) << 24;
		value |= (buffer[offset + 1] & 0xFF) << 16;
		value |= (buffer[offset + 2] & 0xFF) << 8;
		value |= (buffer[offset + 3] & 0xFF);
		return value;
	}

	public static long readLong(byte[] buffer, int offset)
			throws BufferAccessException {
		return ((long) readInt(buffer, offset) << 32)
				| (readInt(buffer, offset + 4) & 0xffffffffL);
	}

	public static short readShort(byte[] buffer, int offset)
			throws BufferAccessException {
		int val = (buffer[offset] & 0xFF) << 8;
		val |= (buffer[offset + 1] & 0xFF);
		return ((short) val);
	}

	public static String readString(byte[] buffer, int offset)
			throws BufferAccessException {
		int strLength = readShort(buffer, offset);
		return new String(buffer, offset + 2, strLength);
	}

	public static void writeBoolean(boolean v, byte[] buffer, int offset)
			throws BufferAccessException {
		if (v == true)
			buffer[offset] = (true ? 1 : 0);
	}

	public static void writeByte(int v, byte[] buffer, int offset)
			throws BufferAccessException {
		buffer[offset] = (byte) v;
	}

	public static void writeChar(int v, byte[] buffer, int offset)
			throws BufferAccessException {
		buffer[offset] = (byte) ((v & 0xFF00) >> 8);
		buffer[offset + 1] = (byte) ((v & 0x00FF));
	}

	public static void writeDouble(double v, byte[] buffer, int offset)
			throws BufferAccessException {
		writeLong(Double.doubleToLongBits(v), buffer, offset);
	}

	public static void writeFloat(float v, byte[] buffer, int offset)
			throws BufferAccessException {
		writeInt(Float.floatToIntBits(v), buffer, offset);
	}

	public static void writeInt(int v, byte[] buffer, int offset)
			throws BufferAccessException {

		buffer[offset] = (byte) (0xFF & (v >> 24));
		buffer[offset + 1] = (byte) (0xFF & (v >> 16));
		buffer[offset + 2] = (byte) (0xFF & (v >> 8));
		buffer[offset + 3] = (byte) (0xFF & v);
	}

	public static void writeLong(long v, byte[] buffer, int offset)
			throws BufferAccessException {
		int hiInt = (int) (v >> 32);
		int loInt = (int) (v & 0xFFFFFFFF);

		writeInt(hiInt, buffer, offset);
		writeInt(loInt, buffer, offset + 4);
	}

	public static void writeShort(int v, byte[] buffer, int offset)
			throws BufferAccessException {
		buffer[offset] = (byte) (0xFF & (v >> 8));
		buffer[offset + 1] = (byte) (0xFF & v);
	}

	public static void writeString(String str, byte[] buffer, int offset)
			throws BufferAccessException {
		byte[] strBuf = str.getBytes();
		int strLength = strBuf.length;
		writeShort(strLength, buffer, offset);
		for (int i = 0; i < strLength; i++)
			buffer[offset + i + 2] = strBuf[i];
	}


    //------------------------------------------------------------
    public static String bufferToString(byte[] buffer, int offset, int length) {
        StringBuffer result = new StringBuffer();
        int max = (length+offset>buffer.length?buffer.length:length+offset);
        for (int i=offset; i<max; i++)
            result.append(hexa(buffer[i]) + " ");
        return result.toString();
    }

    //------------------------------------------------------------
    public static String hexa(byte v) {
        int low  = v & 0x0F;
        int high = (v & 0xF0) >> 4;
        return "0x" + String.valueOf(digits[high]) + String.valueOf(digits[low]);
    }

}
