/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.util;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.InetSocketAddress;

public class FileTransfer {

    // ==================================================
    public class FileReceiver extends Thread {

	private Socket socket;
	private File dir;

	// --------------------------------------------------
	public FileReceiver(Socket socket, File dir) {
	    this.socket = socket;
	    this.dir    = dir;
	}

	// --------------------------------------------------
	private void receiveFile(InputStream in, File dir) 
	    throws Exception {
	    
	    byte[] buffer = new byte[1024];
	    
	    // Receiving file name
	    in.read(buffer, 0, 2);
	    short nameLength = DataBufferizer.readShort(buffer, 0);
	    in.read(buffer, 0, nameLength);
	    String fname = new String(buffer, 0, nameLength);
	    
	    File file = new File(dir, fname);
	    OutputStream out = new FileOutputStream(file);
	    
	    // Creating file
	    inToOut(in, out);
	    
	    // Closing streams
	    in.close();
	    out.flush();
	    out.close();
	    System.out.println("Received file " + file);
	}
	
	// ------------------------------------------------------------
	public void run() {

	    try {
		receiveFile(socket.getInputStream(), dir);
	    }
	    catch (Exception e) {
		System.err.println("Transmission failed on socket " + socket);
	    }
	}
	    
    }
    // ==================================================

    ServerSocket socket_;
    File dir_;
    // --------------------------------------------------
    public FileTransfer(int port, File dir)
	throws Exception {
	
	if (! dir.exists())
	    dir.mkdirs();
	
	dir_ = dir;
	socket_ = new ServerSocket(port);
    }

    // ------------------------------------------------------------
    public void run() {
	
	Socket clientSocket = null;
	while(! socket_.isClosed()) {
	    try {
		clientSocket = socket_.accept();
		
		FileReceiver receiver = new FileReceiver(clientSocket, dir_);
		receiver.start();
	    }
	    catch (Exception e) {
		System.err.println("Transmission failed on socket " + clientSocket);
	    }
	}
    }

    // ------------------------------------------------------------
    private static void inToOut(InputStream in, OutputStream out) {

	byte[] buffer = new byte[1024];

	try {
	    int nr = in.read(buffer);
	    while (nr > 0) {
		if (nr > 0)
		    out.write(buffer, 0, nr);
		nr = in.read(buffer);
	    }
	}
	catch (Exception e) {}
    }

    // ------------------------------------------------------------
    public static void sendFile(File file,
				InetSocketAddress sockAddr)
	throws Exception {

	Socket socket = new Socket();
	socket.connect(sockAddr, 2000);

	OutputStream out = socket.getOutputStream();
	
	InputStream in = new FileInputStream(file);
	int fileLength = (int) file.length();
	String fname = file.getName();
	
	byte[] buffer = new byte[1024];
	// Sending file name
	DataBufferizer.writeString(fname, buffer, 0);
	out.write(buffer, 0, fname.length() + 2);

	// Sending file content
	inToOut(in, out);

	// Closing streams
	in.close();
	out.flush();
	out.close();
	System.out.println("File " + file + " sent to " + sockAddr);
    }

    // ------------------------------------------------------------
    public static void main(String[] args) throws Exception {

	if (args.length != 2) {
	    System.err.println("Syntax: FileTransfer [directory] [port]");
	    System.exit(1);
	}
	
	File dir = new File(args[0]);
	int port = Integer.parseInt(args[1]);
	
	FileTransfer server = new FileTransfer(port, dir);
	server.run();
    }
	    
	
}
