/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;


/**
 * A <code>Selector&lt;E&gt;</code> can be used to check if an object of type
 * <code>E</code> matches a given selection criterion. Actual criteria must be
 * defined in classes that implement this interface.
 * 
 * @author $Author: F. Guidec$
 * @version $Revision: 1.0$
 */

public interface Selector<E> {

	/**
	 * Checks if the selection criterion defined in this selector is matched by
	 * an object of type <code>E</code>
	 * 
	 * @param value
	 *            a reference to the object that must be matched against the
	 *            selection criterion defined in this <code>Selector</code>
	 * 
	 * @return <code>true</code> if the object matches the criterion of this
	 *         <code>Selector</code>, <code<false</code> otherwise
	 * @throws REException 
	 */
	public boolean isMatchedBy(E value);

}
