/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

import java.util.Random;
import java.math.BigInteger;

public class StringGenerator {

    static protected StringGenerator generator_;
    
    private int counter_ = 0;
    private String seed_;


    // ------------------------------------------------------------
    static public StringGenerator getGenerator() {
	if (generator_ == null)
	    generator_ = new StringGenerator();
	return generator_;
    }
    
    // ------------------------------------------------------------
    static public void setSeed(String seed) {
	generator_ = new StringGenerator(seed);
    }
    
    // ------------------------------------------------------------
    public StringGenerator() {
	
	Random random = new Random(RandomSeed.getSeed());
	BigInteger bigInt = new BigInteger(64, random);
	seed_ = bigInt.toString(16);
    }
    
    // ------------------------------------------------------------
    public StringGenerator(String seed) {
	
	seed_ = seed;
    }
    
    // ------------------------------------------------------------
    synchronized public String newString() {
	String result = seed_ + "_" + counter_;
	counter_++;
	return result;
    }
    
    // ------------------------------------------------------------
    public void setGenerator(StringGenerator generator) {
	generator_ = generator;
    }
}
